@@ -1,9 +1,9 @@
 /* 
  * ISO 8859-1 and UTF-8 to Morse code mappings 
  * This file is part of ebook2cw and may be used under the same terms (GPL)
- * (C) 2008 Fabian Kurz, DJ1YFK 
+ * (C) 2008-2017 Fabian Kurz, DJ1YFK 
  *
- * $Id: codetables.h 209 2011-04-13 18:10:59Z dj1yfk $
+ * $Id$
  *
  * Currently included: ISO 8859-1
  *                      - complete (all that can somehow be mapped)
@@ -13,7 +13,7 @@
  *                      - Greek
  *                      - Hebrew
  *
- * Any comments, additions or corrections are welcome: mail@fkurz.net 
+ * Any comments, additions or corrections are welcome: fabian@fkurz.net 
  *
  */
 
@@ -135,7 +135,7 @@ const static char *iso8859[] = {
 	".-",		/* ~A */
 	".-.-",		/* "A */
 	".--.-",	/* A */
-	".- .",		/* AE */
+	".-.-",		/* AE */
 	"-.-..",	/* ,C */
 	"..-..",	/* `E */
 	"..-..",	/* 'E */
@@ -167,7 +167,7 @@ const static char *iso8859[] = {
 	".-",		/* ~a */
 	".-.-",		/* "a */
 	".--.-",	/* a */
-	".- .",		/* ae */
+	".-.-",		/* ae */
 	"-.-..",	/* ,c */
 	"..-..",	/* `e */
 	"..-..",	/* 'e */
@@ -227,7 +227,7 @@ const static char *utf8table[] = {
 	".-",		/* ~A */
 	".-.-",		/* "A */
 	".--.-",	/* A */
-	".- .",		/* AE */
+	".-.-",		/* AE */
 	"-.-..",	/* ,C */
 	"..-..",	/* `E */
 	"..-..",	/* 'E */
@@ -259,7 +259,7 @@ const static char *utf8table[] = {
 	".-",		/* ~a */
 	".-.-",		/* "a */
 	".--.-",	/* a */
-	".- .",		/* ae */
+	".-.-",		/* ae */
 	"-.-..",	/* ,c */
 	"..-..",	/* `e */
 	"..-..",	/* 'e */
@@ -406,10 +406,12 @@ const static char *utf8table[] = {
 	"-.-.", "..", "-.-", ".-..", "--", "-.", "-..-", "---", 
 	".--.", ".-.", NULL, "...", "-", "-.--", "..-.", "----", 
 	"--.-", ".--", NULL, NULL, NULL, NULL, NULL, NULL, 
-	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
-	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
-	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
-	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
+	NULL,
+	/* Greek Alphabet, small letters */       
+	".-", "-..." , "--.", "-..", ".", "--..", "....", 
+	"-.-.", "..", "-.-", ".-..", "--", "-.", "-..-", "---", 
+	".--.", ".-.", NULL, "...", "-", "-.--", "..-.", "----", 
+	"--.-", ".--", NULL, NULL, NULL, NULL, NULL, NULL, 
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
