@@ -1,9 +1,9 @@
 /* 
-ebook2cw - converts an ebook to morse mp3s
+ebook2cw - converts an ebook to Morse MP3/OGG-files
 
-Copyright (C) 2007 - 2012  Fabian Kurz, DJ1YFK
+Copyright (C) 2007 - 2020  Fabian Kurz, DJ1YFK
 
-$Id: ebook2cw.c 559 2013-01-04 14:14:36Z dj1yfk $
+https://fkurz.net/ham/ebook2cw.html
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -35,6 +35,21 @@ source code looks properly indented with ts=4
 #include <ctype.h>
 #include <time.h>
 
+#if !__MINGW32__
+#include <locale.h>			/* For GNU gettext */
+#include <libintl.h>
+#define _(String) gettext (String)
+#define gettext_noop(String) String
+#define N_(String) gettext_noop (String)
+#else
+#define _(String) (String)
+#endif
+
+#ifndef CGI
+#include <signal.h>			/* Ctrl-C handling with signalhandler() */
+#include <setjmp.h>			/* longjmp */
+#endif
+
 /* for mkdir, not used on Windows */
 #if !__MINGW32__
 #include <sys/stat.h>
@@ -48,9 +63,9 @@ source code looks properly indented with ts=4
 #define VERSION "0.0.0"
 #endif
 
-#define PCMBUFFER   220500	/* 20 seconds at 11kHz, for one word. plenty.	*/
-#define NOISEBUFFER 220500	/* 20 seconds at 11kHz, for one word. plenty.	*/
-#define MP3BUFFER   285000    /* abt 1.25*PCMBUFFER + 7200, as recommended	*/
+#define PCMBUFFER   1048576   /* 90 seconds at 11kHz, for one word. plenty.	*/
+#define NOISEBUFFER 1048576   /* 90 seconds at 11kHz, for one word. plenty.	*/
+#define MP3BUFFER   1310720   /* abt 1.25*PCMBUFFER + 7200, as recommended	*/
 
 #define ISO8859 0
 #define UTF8    1
@@ -82,8 +97,12 @@ vorbis_dsp_state vd;
 vorbis_block     vb;
 #endif
 
-/* Struct CWP to keep all CW parameters */
+#ifndef CGI
+/* Globals for longjmp */
+static jmp_buf jmp;
+#endif
 
+/* Struct CWP to keep all CW parameters */
 typedef struct {
 	/* CW parameters */
 	int wpm, 					/* speed, words per minute */
@@ -121,8 +140,9 @@ typedef struct {
 			*noisebuf;
 	unsigned char *mp3buffer;
 	/* Chapter splitting */
-	char chapterstr[80], 		/* split chapters by this string */
-		 chapterfilename[80];
+	char chapterstr[80], 				/* split chapters by this string */
+		 chapterfilename[80],			/* Prefix, e.g. "Chapter-" */
+		 outfilename[256];				/* Full name of current outputfile */
 	/* time based splitting, seconds */
 	int chaptertime;
 	/* word based splitting */
@@ -136,7 +156,15 @@ typedef struct {
 	char isomap[256][4]; 		/* by these strings */
 	char utf8map[256][8];
 
-	char configfile[1025];
+	int comment;				/* status for comments (which will not
+								   be translated to CW), 0 = off, 1 = on,
+								   2 = off after next word */
+
+	/* Line of input file; position (byte) within the line */
+	int linecount;
+	int linepos;
+
+	char configfile[2048];
 
 	char id3_author[80],
 		id3_title[80],
@@ -150,6 +178,7 @@ typedef struct {
 
 /* functions */
 void  init_cw (CWP *cw);
+void  init_cwp (CWP *cw);
 void  init_encoder (CWP *cw);
 void  encode_buffer (int length, CWP *cw);
 void  ogg_encode_and_write (CWP *cw);
@@ -175,7 +204,10 @@ void  scalebuffer(short int *buf, int length, float factor);
 void  addbuffer (short int *b1, short int *b2, int l);
 char  *timestring (int ms);
 void  guessencoding (char *filename);
-void add_silence (int ms, CWP *cw);
+void  add_silence (int ms, CWP *cw);
+#ifndef CGI
+void  signalhandler(int signal);
+#endif
 
 #ifdef CGI
 int   hexit (char c);
@@ -188,16 +220,18 @@ int main (int argc, char** argv) {
 	int pos, i, c, tmp;
  	char word[1024]="";				/* will be cut when > 1024 chars long */
 	int chapter = 0;
-	int chw = 0, tw = 0, qw = 0;	/* chapter words, total words, qrq words */
+    int download = 0;               /* CGI: Send content-disposition header? */
+	int chw = 0, tw = 0;			/* chapter words, total words */
 	int chms = 0, tms = 0, qms = 0; /* millisec: chapter, total, since qrq */
 	time_t start_time, end_time;	/* conversion time */
 	int finishchapter = 0;			/* finish chapter after this sentence */
+	int interactive = 0;			/* 0 for pipe/file input, 1 for tty */
 	FILE *infile;
 
 #ifdef CGI
-	char * querystring;
+	char *querystring;
 	static char text[10000];
-	static char outfilename[1024];
+	static char cgi_outfilename[1024];
 #endif
 
 #ifdef CGIBUFFERED
@@ -206,61 +240,31 @@ int main (int argc, char** argv) {
 
 	/* initializing the CW parameter struct with standard values */
 	CWP cw;
-	cw.wpm = 25;
-	cw.freq = 600;
-	cw.rt = 50;
-	cw.ft = 50;
-	cw.qrq = 0;
-	cw.reset = 1;
-	cw.farnsworth = 0;
-	cw.ews = 0.0;
-	cw.pBT = 1;
-	cw.waveform = SINE;
+	init_cwp(&cw);
 	
-	cw.bandpassbw = 500;
-	cw.bandpassfc = 800;
-	cw.addnoise = 0;
-	cw.snr = 0;
-
-#ifdef LAME
-	cw.encoder = MP3;
-#else
-	cw.encoder = OGG;
-#endif
-
-	cw.samplerate = 11025;
-	cw.brate = 16;
-	cw.quality = 5;
-	
-	cw.inpcm_size = PCMBUFFER;
-	cw.noisebuf_size = NOISEBUFFER;
-	cw.mp3buffer_size = MP3BUFFER;
-	cw.ditlen = 0;
-	strcpy(cw.chapterstr, "CHAPTER");
-	strcpy(cw.chapterfilename, "Chapter");
-
-	cw.chaptertime = 0;
-	cw.chapterwords = 0;
-
-	cw.encoding = ISO8859;
-	cw.use_isomapping = cw.use_utf8mapping = 0;
-
-	strcpy(cw.configfile, "ebook2cw.conf");
-
-	strcpy(cw.id3_author, "CW audio book");
-	strcpy(cw.id3_title, "");
-	strcpy(cw.id3_comment, "Generated by ebook2cw");
-	strcpy(cw.id3_year, "");
-
 	infile = stdin;
-	cw.outfile_length = 0;
 	
 	start_time = time(NULL);
 	srand((unsigned int) start_time);
 
-#ifndef CGI
+#if !__MINGW32__
+	/* Native Language Support by GNU gettext */
+	setlocale(LC_ALL, "" );
+	bindtextdomain( "ebook2cw", "/usr/share/locale" );
+	textdomain("ebook2cw");
+#endif
 
-	printf("ebook2cw %s - (c) 2013 by Fabian Kurz, DJ1YFK\n\n", VERSION);
+#ifndef CGI
+	/* Signal handling for Ctrl-C -- Does not work on Win32 because
+	 * SIGINT is not supported on that platform. There, the signal
+	 * handler will be called but then the program terminates. */
+
+	if (signal(SIGINT, signalhandler) == SIG_ERR) {
+		fprintf(stderr, _("Failed to set up signal handler for SIGINT\n"));	
+		return EXIT_FAILURE;
+	}
+
+	printf(_("ebook2cw %s - (c) 2007 - 2020 by Fabian Kurz, DJ1YFK\n\n"), VERSION);
 
 	/* 
 	 * Find and read ebook2cw.conf 
@@ -275,12 +279,31 @@ int main (int argc, char** argv) {
 	if (optind < argc) {		/* something left? if so, use as infile */
 		if ((argv[optind][0] != '-') && (argv[optind][0] != '\0')) {
 			if ((infile = fopen(argv[optind], "r")) == NULL) {
-				fprintf(stderr, "Error: Cannot open file %s. Exit.\n", 
+				fprintf(stderr, _("Error: Cannot open file %s. Exit.\n"), 
 								argv[optind]);
 				exit(EXIT_FAILURE);
 			}
 		}
 	}
+
+	/* If we are getting data from a tty, check what the actual encoding is.
+	 * Historically, the default encoding of ebook2cw is ISO 8859-1, but
+	 * nowadays most terminals should use UTF-8. */
+	if (isatty(fileno(infile))) {
+		interactive = 1;
+#if !__MINGW32__
+		if (getenv("LANG") != NULL) {
+			if (strstr(getenv("LANG"), "utf") || 
+				strstr(getenv("LANG"), "UTF")) {
+				cw.encoding = UTF8;
+			}
+		}
+#else
+		/* Assume Windows terminal to be UTF8 */
+		cw.encoding = UTF8;
+#endif
+	}
+
 #endif	/* ifndef CGI */
 
 	/* init encoder (LAME or OGG/Vorbis) */
@@ -290,26 +313,32 @@ int main (int argc, char** argv) {
 	buf_alloc(&cw);
 
 #ifndef CGI
-	printf("Speed: %dwpm, Freq: %dHz, Chapter: >%s<, Encoding: %s\n", cw.wpm, 
+	printf(_("Speed: %dwpm, Freq: %dHz, Chapter: >%s<, Encoding: %s\n"),cw.wpm, 
 		cw.freq, cw.chapterstr, cw.encoding == UTF8 ? "UTF-8" : "ISO 8859-1");
 
-	printf("Effective speed: %dwpm, ", cw.farnsworth ? cw.farnsworth : cw.wpm);
-	printf("Extra word spaces: %1.1f, ", cw.ews);
-	printf("QRQ: %dmin, reset QRQ: %s\n", cw.qrq, cw.reset ? "yes" : "no");
+	printf(_("Effective speed: %dwpm, "), cw.farnsworth ? cw.farnsworth : cw.wpm);
+	printf(_("Extra word spaces: %1.1f, "), cw.ews);
+	printf(_("QRQ: %dmin, reset QRQ: %s\n"), cw.qrq, cw.reset ? _("yes") : _("no"));
 
 	if (cw.chapterwords) {
-		printf("Chapter limit: %d words, ", cw.chapterwords);
+		printf(_("Chapter limit: %d words, "), cw.chapterwords);
 	}
 	if (cw.chaptertime) {
-		printf("Chapter limit: %d seconds, ", cw.chaptertime);
+		printf(_("Chapter limit: %d seconds, "), cw.chaptertime);
 	}
 
-	printf("Encoder: %s\n\n", (cw.encoder == OGG) ? "OGG" : "MP3");
+	printf(_("Encoder: %s\n\n"), (cw.encoder == OGG) ? "OGG" : "MP3");
+
+	if (interactive) {
+		printf(_("Interactive mode. Type in text and finish with Ctrl-D.\n\n"));
+	}
 
 #endif
 
 
 #ifdef CGI
+/* CGI: utf8 input by default */
+cw.encoding = UTF8;
 /* CGI standard: write directly to stdout ==> no Content-Length possible
  * CGI buffered: write to file /tmp/$time-$rnd and later generate output */
 #ifndef CGIBUFFERED				
@@ -319,7 +348,7 @@ int main (int argc, char** argv) {
 	 * just using rand(time) won't do if two requests come in the
 	 * same second. temporarily use cw.outfile FD to open urand */
 	if ((cw.outfile = fopen("/dev/urandom", "r")) == NULL) {
-	        fprintf(stderr, "Error: Failed to open /dev/urandom.\n");
+	        fprintf(stderr, _("Error: Failed to open /dev/urandom.\n"));
 	        exit(EXIT_FAILURE);
 	}
 	fread(&i, sizeof(i), 1, cw.outfile);
@@ -327,9 +356,9 @@ int main (int argc, char** argv) {
 	i = rand();
 	fclose(cw.outfile);
 
-	snprintf(outfilename, 80, "/tmp/%d-%d", start_time, i);
-	if ((cw.outfile = fopen(outfilename, "wb+")) == NULL) {
-	        fprintf(stderr, "Error: Failed to open %s\n", outfilename);
+	snprintf(cgi_outfilename, 80, "/tmp/%d-%d", (int)start_time, i);
+	if ((cw.outfile = fopen(cgi_outfilename, "wb+")) == NULL) {
+	        fprintf(stderr, _("Error: Failed to open %s\n"), cgi_outfilename);
 	        exit(EXIT_FAILURE);
 	}
 #endif
@@ -348,10 +377,25 @@ int main (int argc, char** argv) {
 	if ((querystring == NULL) || strlen(querystring) > 9000) {
 			exit(1);
 	}
-	sscanf(querystring, "s=%d&e=%d&f=%d&t=%9000s", &cw.wpm, &cw.farnsworth, &cw.freq, text);
+
+    /* the query parameters may look like:
+     * d=001&s=25&e=20&f=600&t=text  => download, offer as filename "lcwo-001.mp3"
+     * s=25&e=20&f=600&t=text        => return normal file
+     */
+
+    if (querystring[0] == 'd') {
+	    sscanf(querystring, "d=%d&s=%d&e=%d&f=%d&t=%9000s", &download, &cw.wpm, &cw.farnsworth, &cw.freq, text);
+    }
+    else {
+    	sscanf(querystring, "s=%d&e=%d&f=%d&t=%9000s", &cw.wpm, &cw.farnsworth, &cw.freq, text);
+    }
 	strcat(text, " ");
 	urldecode(text);
 
+    if (cw.wpm == 0 || cw.freq == 0) {
+        exit(1);
+    }
+
 	flush_ogg(&cw);
 #endif
 
@@ -363,6 +407,11 @@ int main (int argc, char** argv) {
 	chapter = 0;
 #ifndef CGI
 	openfile(chapter, &cw);
+	/* Entry point for handling SIGINT; we jump back here from
+	 * the signalhandler() function */
+	if (setjmp(jmp)) {
+		goto cleanup;
+	}
 #endif
 
 	i=0;
@@ -386,15 +435,31 @@ int main (int argc, char** argv) {
 		if (c == '\r') 			/* DOS linebreaks */
 				continue;
 
-		word[pos++] = c;
+#if __MINGW32__
+		if (c == 0x04)			/* EOT; Win32 console produces this for Ctl-D */
+				break;
+#endif
 
+		if (c == '#') 
+			cw.comment = 1;
+
+		if (cw.comment) {
+		   if (c == '\n') {
+				cw.comment = 2;
+		   }
+		   else {
+			   continue;
+		   }
+		}
+
+		word[pos++] = c;
 		
 		if ((c == ' ') || (c == '\n') || (pos == 1024)) {
 			word[pos] = '\0';
 #ifndef CGI
 			/* new chapter */
 			if ((strcmp(cw.chapterstr, word) == 0) ||	/* regular */
-					finishchapter == 2
+					finishchapter == 2					/* timeout/max. words */
 			   ) {
 
 				closefile(chapter, chw, chms, &cw);
@@ -407,7 +472,6 @@ int main (int argc, char** argv) {
 				if (cw.qrq && cw.reset) {	
 					cw.wpm = cw.original_wpm;	
 					init_cw(&cw);
-					qw = 0;
 				}
 
 				finishchapter = 0;
@@ -425,6 +489,8 @@ int main (int argc, char** argv) {
 				chms += tmp;
 				qms += tmp;
 				chw++; 
+				if (cw.comment == 2) 
+					cw.comment = 0;
 			}
 
 			/* Every 'cw.qrq' minutes speed up 1 WpM */
@@ -458,19 +524,16 @@ int main (int argc, char** argv) {
 
 	} /* eof */
 
-
 /* CGI: Add some silence (500ms) to the end of the file */
 #ifdef CGI
 	add_silence(500, &cw);
 #endif
 
-
-
 #ifndef CGI
 	closefile(chapter, chw, chms, &cw);
 	end_time = time(NULL);
-	printf("Total words: %d, total time: %s\n", tw+chw, timestring(tms+chms));
-	printf("Conversion time: %s (Speedup: %.1fx)\n", 
+	printf(_("Total words: %d, total time: %s\n"), tw+chw, timestring(tms+chms));
+	printf(_("Conversion time: %s (Speedup: %.1fx)\n"), 
 			timestring(1000.0 * difftime(end_time, start_time)), 
 			((tms+chms)/(1000.0 * difftime(end_time,start_time)))
 	);
@@ -501,6 +564,12 @@ int main (int argc, char** argv) {
 	free(cw.mp3buffer);
 	free(cw.inpcm);
 	free(cw.noisebuf);
+	free(cw.dah_buf);
+	free(cw.dit_buf);
+
+    if (download) {
+	    printf("Content-Disposition: attachment; filename=\"lcwo-%03d.mp3\"\n", download);
+    }
 
 #ifdef CGIBUFFERED
 	/* File is completed, so we know the length and can send the 
@@ -514,14 +583,25 @@ int main (int argc, char** argv) {
 	 * easier and faster... */
 	cgibuf = malloc((size_t) i+1);
 	if (cgibuf == NULL) {
-		fprintf(stderr, "malloc() for cgibuf failed!\n");
+		fprintf(stderr, _("malloc() for cgibuf failed!\n"));
 		exit(EXIT_FAILURE);
 	}
 	fread(cgibuf, sizeof(char), (size_t) i, cw.outfile);
 	fclose(cw.outfile);
 	fwrite(cgibuf, sizeof(char), (size_t) i, stdout);
+	free(cgibuf);
 
-	unlink(outfilename);
+	unlink(cgi_outfilename);
+#endif
+
+	/* NON-CGI operation: If we produced no output, remove the
+	 * empty file */
+#ifndef CGI
+cleanup:
+	if (!chw) {
+		printf(_("Deleting empty file: %s\n"), cw.outfilename);
+		unlink(cw.outfilename);
+	}
 #endif
 
 	return (EXIT_SUCCESS);
@@ -540,8 +620,8 @@ void init_cw (CWP *cw) {
 	/* calculate farnsworth length samples */
 	if (cw->farnsworth) {
 		if (cw->farnsworth > cw->wpm) {
-			fprintf(stderr, "Warning: Effective speed (-e %d) must be lower "
-							"than character speed (-w %d)! Speed adjusted.\n", 
+			fprintf(stderr, _("Warning: Effective speed (-e %d) must be lower "
+							"than character speed (-w %d)! Speed adjusted.\n"), 
 							cw->farnsworth, cw->wpm);
 			cw->farnsworth = cw->wpm;
 		}
@@ -554,11 +634,11 @@ void init_cw (CWP *cw) {
 	/* size of dit_buf, dah_buf may have to be increased, when speed decreased */
 	if (len > cw->ditlen) {
 		if ((cw->dah_buf = realloc(cw->dah_buf, 3*len * sizeof(short int))) == NULL) {
-			fprintf(stderr, "Error: Can't reallocate dah_buf[%d]\n", 3*len);
+			fprintf(stderr, _("Error: Can't reallocate dah_buf[%d]\n"), 3*len);
 			exit(EXIT_FAILURE);
 		}
 		if ((cw->dit_buf = realloc(cw->dit_buf, len * sizeof(short int))) == NULL) {
-			fprintf(stderr, "Error: Can't allocate dit_buf[%d]\n", len);
+			fprintf(stderr, _("Error: Can't allocate dit_buf[%d]\n"), len);
 			exit(EXIT_FAILURE);
 		}
 	}
@@ -638,12 +718,19 @@ int makeword(char * text, CWP *cw) {
  for (i = 0; i < strlen(text); i++) {
 	c = (unsigned char) text[i];
 	code = NULL;
+	cw->linepos++;
 
 	if (c == '\n') { 				/* Same for UTF8 and ISO8859 */
-		if ((strlen(text) == 1) && cw->pBT) 	/* paragraph */
+		if (cw->comment == 0 && strlen(text) == 1 && cw->pBT) 	/* paragraph */
 			code = " -...- "; 
-		else 
+		else if (cw->comment) {		/* No spaces after comment line */
 			code = " ";
+		}
+		else {						/* Space instead of newline */
+			code = " ";
+		}
+		cw->linepos = 0;
+		cw->linecount++;
 	}
 	else if (c == '<') {				/* prosign on */
 		prosign = 1;
@@ -679,12 +766,18 @@ int makeword(char * text, CWP *cw) {
 	/* Not found anything, produce warning message */
 	if (code == NULL) {
 #ifndef CGI
-		if (c < 255) {
-			fprintf(stderr, "Warning: don't know CW for '%c'\n", c);
+		if (c < 128) {
+			fprintf(stderr, _("Warning: Don't know CW for '%c'. "), c);
+		}
+		else if (cw->encoding == ISO8859) {
+			fprintf(stderr, _("Warning: Don't know CW for '%c' (0x%01X) (try the -u switch to enable UTF-8). "), c, c);
 		}
 		else {
-			fprintf(stderr, "Warning: don't know CW for unicode &#%d;\n", c);
+			/* TODO: Consider using libunistring's unicode_character_name()
+			 * function to provide the name of the character */
+			fprintf(stderr, _("Warning: Don't know CW for unicode code point U+%04X. "), c);
 		}
+		fprintf(stderr, _("[Line %d, Byte %d]\n"), cw->linecount, cw->linepos);
 #endif
 		code = " ";
 
@@ -738,17 +831,10 @@ int makeword(char * text, CWP *cw) {
 /* closefile -- finishes writing the current file, flushes the encoder buffer */
 
 void closefile (int chapter, int chw, int chms, CWP *cw) {
-	int outbytes;
-	char outfilename[80] = "";
+	int outbytes = 0;
 
-	outbytes = 0;
-
-	printf("words: %d, time: %s\n", chw, timestring(chms));
-	
-
-	snprintf(outfilename, 80, "%s%04d.%s",  cw->chapterfilename, chapter, 
-			(cw->encoder == MP3) ? "mp3" : "ogg");
-	printf("Finishing %s\n\n",  outfilename);
+	printf(_("words: %d, time: %s\n"), chw, timestring(chms));
+	printf(_("Finishing %s\n\n"),  cw->outfilename);
 
 	switch (cw->encoder) {
 		case MP3:
@@ -783,13 +869,13 @@ void closefile (int chapter, int chw, int chms, CWP *cw) {
 
 	if (cw->encoder != NOENC) 
 		fclose(cw->outfile);
+
 }
 
 
 /* openfile -- starts a new chapter by opening a new file as outfile */
 
 void openfile (int chapter, CWP *cw) {
-	char outfilename[80] = "";
 #ifdef LAME
 	static char tmp[80] = "";
 #endif
@@ -799,13 +885,13 @@ void openfile (int chapter, CWP *cw) {
 	ogg_packet       hdr_code;
 #endif
 
-	snprintf(outfilename, 80, "%s%04d.%s",  cw->chapterfilename, chapter, 
+	snprintf(cw->outfilename, 80, "%s%04d.%s",  cw->chapterfilename, chapter, 
 			(cw->encoder == MP3) ? "mp3" : "ogg");
-	printf("Starting %s\n",  outfilename);
+	printf(_("Starting %s\n"),  cw->outfilename);
  
 	if ((cw->encoder != NOENC) && 
-			(cw->outfile = fopen(outfilename, "wb")) == NULL) {
-		fprintf(stderr, "Error: Failed to open %s\n", outfilename);
+			(cw->outfile = fopen(cw->outfilename, "wb")) == NULL) {
+		fprintf(stderr, _("Error: Failed to open %s\n"), cw->outfilename);
 		exit(EXIT_FAILURE);
 	}
 
@@ -844,19 +930,19 @@ void openfile (int chapter, CWP *cw) {
 
 
 void help (void) {
-	printf("This is free software, and you are welcome to redistribute it\n"); 
-	printf("under certain conditions (see COPYING).\n");
+	printf(_("This is free software, and you are welcome to redistribute it\n")); 
+	printf(_("under certain conditions (see COPYING).\n"));
 	printf("\n");
-	printf("ebook2cw [-w wpm] [-f freq] [-R risetime] [-F falltime]\n");
-	printf("         [-s samplerate] [-b mp3bitrate] [-q mp3quality] [-p]\n");
-	printf("         [-c chapter-separator] [-o outfile-name] [-Q minutes]\n");
-	printf("         [-a author] [-t title] [-k comment] [-y year]\n");
-	printf("         [-u] [-S ISO|UTF] [-n] [-e eff.wpm] [-W space]\n");
-	printf("         [-N snr] [-B filter bandwidth] [-C filter center]\n");
-	printf("         [-T 0..2] [-g filename] [-l words] [-d seconds]\n");
-	printf("         [infile]\n\n");
-	printf("defaults: 25 WpM, 600Hz, RT=FT=50, s=11025Hz, b=16kbps,\n");
-	printf("          c=\"CHAPTER\", o=\"Chapter\" infile = stdin\n");
+	printf(_("ebook2cw [-w wpm] [-f freq] [-R risetime] [-F falltime]\n"));
+	printf(_("         [-s samplerate] [-b mp3bitrate] [-q mp3quality] [-p]\n"));
+	printf(_("         [-c chapter-separator] [-o outfile-name] [-Q minutes]\n"));
+	printf(_("         [-a author] [-t title] [-k comment] [-y year]\n"));
+	printf(_("         [-u] [-S ISO|UTF] [-n] [-e eff.wpm] [-W space]\n"));
+	printf(_("         [-N snr] [-B filter bandwidth] [-C filter center]\n"));
+	printf(_("         [-T 0..2] [-g filename] [-l words] [-d seconds]\n"));
+	printf(_("         [infile]\n\n"));
+	printf(_("defaults: 25 WpM, 600Hz, RT=FT=50, s=11025Hz, b=16kbps,\n"));
+	printf(_("          c=\"CHAPTER\", o=\"Chapter\" infile = stdin\n"));
 	printf("\n");
 	printf("Compile options: USE_LAME="
 #ifdef LAME
@@ -872,7 +958,7 @@ void help (void) {
 #endif
 		"DESTDIR="DESTDIR"\n");
 	printf("\n");
-	printf("Project website: http://fkurz.net/ham/ebook2cw.html\n");
+	printf(_("Project website: http://fkurz.net/ham/ebook2cw.html\n"));
 	exit(EXIT_SUCCESS);
 }
 
@@ -928,7 +1014,8 @@ void buf_check (int j, CWP *cw) {
 			cw->inpcm_size *=  2;
 			cw->noisebuf_size = cw->inpcm_size;
 			cw->mp3buffer_size *= 2;
-			if ((cw->inpcm = realloc(cw->inpcm, cw->inpcm_size*sizeof(short int)))== NULL) {
+
+			if ((cw->inpcm = realloc(cw->inpcm, cw->inpcm_size*sizeof(short int))) == NULL) {
 				fprintf(stderr, "Error: Can't realloc inpcm[%d]\n", cw->inpcm_size);
 				exit(EXIT_FAILURE);
 			}
@@ -939,10 +1026,8 @@ void buf_check (int j, CWP *cw) {
 			}
 			fillnoisebuffer(cw->noisebuf, cw->noisebuf_size, NOISEAMPLITUDE);
 			
-			if ((cw->mp3buffer = realloc(cw->mp3buffer, cw->mp3buffer_size*sizeof(char))) 
-						   	== NULL) {
-				fprintf(stderr, "Error: Can't realloc mp3buffer[%d]\n", 
-								cw->mp3buffer_size);
+			if ((cw->mp3buffer = realloc(cw->mp3buffer, cw->mp3buffer_size*sizeof(char))) == NULL) {
+				fprintf(stderr, "Error: Can't realloc mp3buffer[%d]\n", cw->mp3buffer_size);
 				exit(EXIT_FAILURE);
 			}
 		}
@@ -968,7 +1053,7 @@ void command (char * cmd, CWP *cw) {
 				init_cw(cw);
 			}
 			else 
-				fprintf(stderr, "Invalid frequency: %s. Ignored.\n", cmd);
+				fprintf(stderr, _("Invalid frequency: %s. Ignored.\n"), cmd);
 			break;
 		case 'w':
 			if (i > 1) {
@@ -976,7 +1061,7 @@ void command (char * cmd, CWP *cw) {
 				init_cw(cw);
 			}
 			else 
-				fprintf(stderr, "Invalid speed: %s. Ignored.\n", cmd);
+				fprintf(stderr, _("Invalid speed: %s. Ignored.\n"), cmd);
 			break;
 		case 'e':
 			if (i > 1) {
@@ -984,7 +1069,7 @@ void command (char * cmd, CWP *cw) {
 				init_cw(cw);
 			}
 			else 
-				fprintf(stderr, "Invalid speed: %s. Ignored.\n", cmd);
+				fprintf(stderr, _("Invalid speed: %s. Ignored.\n"), cmd);
 			break;
 		case 'T':
 			if (i >= 0 && i < 3) {
@@ -992,7 +1077,7 @@ void command (char * cmd, CWP *cw) {
 				init_cw(cw);
 			}
 			else 
-				fprintf(stderr, "Invalid waveform: %d. Ignored.\n", i);
+				fprintf(stderr, _("Invalid waveform: %d. Ignored.\n"), i);
 			break;
 		case 'v':
 				init_cw(cw);
@@ -1003,8 +1088,19 @@ void command (char * cmd, CWP *cw) {
 			cw->addnoise = 1;
 			cw->snr = i;
 			break;
+		case 'S':
+			if (i >= 1 && i <= 20000) {
+				add_silence(i, cw);
+			}
+			break;
+		case 'W':
+			if (i >= 0 && i <= 40) {
+				cw->ews = i;
+				init_cw(cw);
+			}
+			break;
 		default:
-			fprintf(stderr, "Invalid command %s. Ignored.\n", cmd);				
+			fprintf(stderr, _("Invalid command %s. Ignored.\n"), cmd);				
 	}
 }
 
@@ -1034,12 +1130,12 @@ void readconfig (CWP *cw) {
 	 */
 
 	if ((conf = fopen(cw->configfile, "r")) == NULL) {
+#if !__MINGW32__
 		/* ebook2cw.conf not in current directory */
 		if ((homedir = getenv("HOME")) == NULL) {	/* no home or Windows */
 			return;
 		}
-#if !__MINGW32__
-		/* Linux/Unix: Look for ~/.ebook2cw/ebook2cw.conf */
+		/* Look for ~/.ebook2cw/ebook2cw.conf */
 		snprintf(cw->configfile, 2048, "%s/.ebook2cw/ebook2cw.conf", homedir);
 		if ((conf = fopen(cw->configfile, "r")) == NULL) {
 			/* Not in ~/.ebook2cw/ either, look for it in
@@ -1057,7 +1153,7 @@ void readconfig (CWP *cw) {
 
 				/* open newly installed config file ... */
 				if ((conf = fopen(cw->configfile, "r")) == NULL) {
-					printf("Couldn't read %s! Continue without config.",
+					printf(_("Couldn't read %s! Continue without config."),
 									cw->configfile);
 					return;
 				}
@@ -1068,6 +1164,8 @@ void readconfig (CWP *cw) {
 				return;
 			}
 		}
+#else
+        return; /* Win32 and no ebook2cw.conf in current dir -> return */
 #endif
 	}
 
@@ -1075,7 +1173,7 @@ void readconfig (CWP *cw) {
 	 * Read config file
 	 */
 
-	printf("Reading configuration file: %s\n\n", cw->configfile);
+	printf(_("Reading configuration file: %s\n\n"), cw->configfile);
 
 	/* We start in the [settings] section.
 	 * All settings are ^[a-zA-Z]=.+$ 
@@ -1112,7 +1210,7 @@ void setparameter (char i, char *value, CWP *cw) {
 		switch (i) {
 			case 'w':
 				if ((cw->wpm = atoi(value)) < 1) {
-					fprintf(stderr, "Error: Speed (-w) must be > 0!\n");
+					fprintf(stderr, _("Error: Speed (-w) must be > 0!\n"));
 					exit(EXIT_FAILURE);
 				}
 				break;
@@ -1160,7 +1258,7 @@ void setparameter (char i, char *value, CWP *cw) {
 				break;
 			case 'Q':
 				if ((cw->qrq = atoi(value)) < 1) {
-					fprintf(stderr, "Error: QRQ time (-Q) must be > 0!\n");
+					fprintf(stderr, _("Error: QRQ time (-Q) must be > 0!\n"));
 					exit(EXIT_FAILURE);
 				}
 				break;
@@ -1177,7 +1275,7 @@ void setparameter (char i, char *value, CWP *cw) {
 				break;
 			case 'e':				/* effective speed in WpM */
 				if ((cw->farnsworth = atoi(value)) < 1) {
-					fprintf(stderr, "Error: Eff. speed (-e) must be > 0!\n");
+					fprintf(stderr, _("Error: Eff. speed (-e) must be > 0!\n"));
 					exit(EXIT_FAILURE);
 				}
 				break;
@@ -1191,7 +1289,7 @@ void setparameter (char i, char *value, CWP *cw) {
 #ifdef OGGV
 				cw->encoder = OGG;
 #else
-				fprintf(stderr, "Warning: ebook2cw was compiled without OGG support! Using Lame encoder.\n\n");
+				fprintf(stderr, _("Warning: ebook2cw was compiled without OGG support! Using Lame encoder.\n\n"));
 #endif
 				break;
 			case 'X':					/* "Fake" */
@@ -1207,13 +1305,13 @@ void setparameter (char i, char *value, CWP *cw) {
 				help();
 				break;
 			case 'T':
-				if (strstr(value, "SINE") || atoi(value) == 0) {
+				if (strstr(value, "SINE") || strstr(value, "0")) {
 					cw->waveform = SINE;
 				}
-				else if (strstr(value, "SAWTOOTH") || atoi(value) == 1) {
+				else if (strstr(value, "SAWTOOTH") || strstr(value, "1")) {
 					cw->waveform = SAWTOOTH;
 				}
-				else if (strstr(value, "SQUARE") || atoi(value) == 2) {
+				else if (strstr(value, "SQUARE") || strstr(value, "2")) {
 					cw->waveform = SQUARE;	
 				}
 				break;
@@ -1221,8 +1319,8 @@ void setparameter (char i, char *value, CWP *cw) {
 				cw->snr = atoi(value);
 				cw->addnoise = 1;
 				if ((cw->snr < -10) || (cw->snr > 10)) {
-					fprintf(stderr, "Warning: SNR %ddB not implemented."
-									" Noise disabled.\n\n", cw->snr);
+					fprintf(stderr, _("Warning: SNR %ddB not implemented."
+									" Noise disabled.\n\n"), cw->snr);
 					cw->addnoise = 0;
 					cw->snr = 0;
 				}
@@ -1256,8 +1354,8 @@ void loadmapping(char *file, int enc, CWP *cw) {
 	char s[6]="";
 
 	if ((mp = fopen(file, "r")) == NULL) {
-		fprintf(stderr, "Warning: Unable to open mapping file %s. Ignored.\n", 
-								file);
+		fprintf(stderr, _("Warning: Unable to open mapping file %s. Ignored.\n")
+				, file);
 		return;
 	}
 
@@ -1301,8 +1399,8 @@ void loadmapping(char *file, int enc, CWP *cw) {
 
 		/* only memory for 256 mappings allocated. never going to happen */
 		if (i == 255) {
-			fprintf(stderr, "Warning: Maximum number (256) of mappings reached"
-							" in %s! Stopped here.", file);
+			fprintf(stderr, _("Warning: Maximum number (256) of mappings "
+						"reached in %s! Stopped here."), file);
 			break;
 		}
 
@@ -1591,6 +1689,7 @@ void filterloop (short int *buf, int l, int b) {
    
    	  buf[k] = (short int) (yv[6] * 255.0);
     }
+  free(in);
 }
 
 
@@ -1777,11 +1876,11 @@ void guessencoding (char *filename) {
 	int is_iso   = 1;
 
 	if ((infile = fopen(filename, "r")) == NULL) {
-		fprintf(stderr, "Error: Cannot open file %s. Exit.\n", filename);
+		fprintf(stderr, _("Error: Cannot open file %s. Exit.\n"), filename);
 		exit(EXIT_FAILURE);
 	}
 
-	printf("Guessed file format of %s: ", filename);
+	printf(_("Guessed file format of %s: "), filename);
 
 	while ((c = getc(infile)) != EOF) {
 		if ((c & 224) == 192) {		/* check if highest 3 bits are 110=224 */
@@ -1844,12 +1943,12 @@ int install_config_files (char *homedir, CWP *cw) {
 	int j = 0;
 	char tmp[1024] = "";
 	
-	printf("First run. Copying example configuration files to "
-		"%s/.ebook2cw/\n\n", homedir);
+	printf(_("First run. Copying example configuration files to "
+		"%s/.ebook2cw/\n\n"), homedir);
 	snprintf(tmp, 1024, "%s/.ebook2cw/", homedir);
 	j = mkdir(tmp, 0777);
 	if (j && (errno != EEXIST)) {
-		printf("Failed to create %s. Resuming without config.",tmp);
+		printf(_("Failed to create %s. Resuming without config."), tmp);
 		return -1;
 	}
 	/* ~/.ebook2cw/ created. Now copy ebook2cw.conf and map files */
@@ -1857,8 +1956,8 @@ int install_config_files (char *homedir, CWP *cw) {
 		"/share/doc/ebook2cw/examples/ebook2cw.conf %s/.ebook2cw/",
 		homedir);
 	if (system(tmp)) {
-		printf("Failed to create ~/.ebook2cw/ebook2cw.conf. "
-						"Resuming without config.");
+		printf(_("Failed to create ~/.ebook2cw/ebook2cw.conf. "
+						"Resuming without config."));
 		return -1;
 	}
 
@@ -1866,8 +1965,8 @@ int install_config_files (char *homedir, CWP *cw) {
 		"/share/doc/ebook2cw/examples/isomap.txt %s/.ebook2cw/",
 		homedir);
 	if (system(tmp)) {
-		printf("Failed to create ~/.ebook2cw/isomap.txt. "
-						"Resuming without config.");
+		printf(_("Failed to create ~/.ebook2cw/isomap.txt. "
+						"Resuming without config."));
 		return -1;
 	}
 
@@ -1875,14 +1974,87 @@ int install_config_files (char *homedir, CWP *cw) {
 		"/share/doc/ebook2cw/examples/utf8map.txt %s/.ebook2cw/",
 		homedir);
 	if (system(tmp)) {
-		printf("Failed to create ~/.ebook2cw/utf8map.txt. "
-						"Resuming without config.");
+		printf(_("Failed to create ~/.ebook2cw/utf8map.txt. "
+						"Resuming without config."));
 		return -1;
 	}
 #endif
 	return 0;
 }
 
+/* We jump here in case of SIGINT. On Win32
+ * the program will terminate anyway, so the
+ * longjmp and cleanup procedure will not
+ * work. */
+#ifndef CGI
+void signalhandler (int signal) {
+#if !__MINGW32__
+	printf(_("Caught SIGINT. Cleaning up.\n"));
+	longjmp(jmp,1);
+#else
+	printf(_("Caught SIGINT. Terminating.\n"));
+#endif
+}
+#endif
+
+/* Initialisation of cw parameter struct */
+void init_cwp (CWP *cw) {
+	cw->wpm = 25;
+	cw->freq = 600;
+	cw->rt = 50;
+	cw->ft = 50;
+	cw->qrq = 0;
+	cw->reset = 1;
+	cw->farnsworth = 0;
+	cw->ews = 0.0;
+	cw->pBT = 1;
+	cw->waveform = SINE;
+	
+	cw->bandpassbw = 500;
+	cw->bandpassfc = 800;
+	cw->addnoise = 0;
+	cw->snr = 0;
+
+#ifdef LAME
+	cw->encoder = MP3;
+#else
+	cw->encoder = OGG;
+#endif
+
+	cw->samplerate = 11025;
+	cw->brate = 16;
+	cw->quality = 5;
+	
+	cw->inpcm_size = PCMBUFFER;
+	cw->noisebuf_size = NOISEBUFFER;
+	cw->mp3buffer_size = MP3BUFFER;
+	cw->ditlen = 0;
+	strcpy(cw->chapterstr, _("CHAPTER"));
+	strcpy(cw->chapterfilename, _("Chapter"));
+
+	cw->chaptertime = 0;
+	cw->chapterwords = 0;
+
+	cw->encoding = ISO8859;
+	cw->use_isomapping = cw->use_utf8mapping = 0;
+
+	cw->comment = 0;
+
+	cw->linecount = 1;
+	cw->linepos = 0;
+
+	strcpy(cw->configfile, "ebook2cw.conf");
+
+	strcpy(cw->id3_author, _("CW audio book"));
+	strcpy(cw->id3_title, "");
+	strcpy(cw->id3_comment, _("Generated by ebook2cw"));
+	strcpy(cw->id3_year, "");
+
+	cw->outfile_length = 0;
+
+}
+
+
 
 /* vim: noai:ts=4:sw=4 
  * */
